// ImageViewController.m

#import "ImageViewController.h"

static int  _maxPage = 10;

@interface ImageViewController (private)

// Appearance
- (void)_renewImages;

@end

@implementation ImageViewController

//--------------------------------------------------------------//
#pragma mark -- View --
//--------------------------------------------------------------//

- (void)viewDidLoad
{
    // innerViewをメインスクロールビューに追加
    [_mainScrollView addSubview:_innerView];
    
    // メインスクロールビューのコンテントサイズを設定
    _mainScrollView.contentSize = _innerView.frame.size;
    
    // インデックスの初期値として-1を設定
    _index = -1;
}

- (void)viewWillAppear:(BOOL)animated
{
    // 親クラスの呼び出し
    [super viewWillAppear:animated];
    
    // 画像の更新
    [self _renewImages];
}

//--------------------------------------------------------------//
#pragma mark -- Image --
//--------------------------------------------------------------//

- (void)_renewImages
{
    CGRect      rect;
    NSString*   fileName;
    NSString*   path;
    UIImage*    image;
    
    // 現在のインデックスを保存
    int oldIndex = _index;
    
    // コンテントオフセットを取得
    CGPoint offset;
    offset = _mainScrollView.contentOffset;
    if (offset.x == 0) {
        // 前の画像へ移動
        _index--;
    }
    if (offset.x >= _mainScrollView.contentSize.width - CGRectGetWidth(_mainScrollView.frame)) {
        // 次の画像へ移動
        _index++;
    }
    
    // インデックスの値をチェック
    if (_index < 0) {
        _index = 0;
    }
    if (_index > _maxPage - 1) {
        _index = _maxPage - 1;
    }
    
    if (_index == oldIndex) {
        return;
    }
    
    //
    // 左側のimage viewを更新
    //
    
    // 最初の画像のとき
    if (_index == 0) {
        // 左側のimage viewは表示しない
        rect = CGRectZero;
        
        image = nil;
    }
    // 最初の画像以外のとき
    else {
        // 左側のimage viewのframe
        rect.origin = CGPointZero;
        rect.size = self.view.frame.size;
        
        // 左側のimage viewの画像の読み込み
        fileName = [NSString stringWithFormat:@"%03ds", _index - 1];
        path = [[NSBundle mainBundle] pathForResource:fileName ofType:@"jpg"];
        image = [[UIImage alloc] initWithContentsOfFile:path];
    }
    
    // 左側のimage viewの設定
    _imageView0.frame = rect;
    _imageView0.image = image;
    [image release];
    
    //
    // 中央のimage view、サブスクロールビューを更新
    //
    
    // 中央のimage viewの画像の読み込み
    fileName = [NSString stringWithFormat:@"%03d", _index];
    path = [[NSBundle mainBundle] pathForResource:fileName ofType:@"jpg"];
    image = [[UIImage alloc] initWithContentsOfFile:path];
    
    // サブスクロールビューのframe
    rect.origin.x = CGRectGetMaxX(_imageView0.frame) > 0 ?
            CGRectGetMaxX(_imageView0.frame) + 20.0f : 0;
    rect.origin.y = 0;
    rect.size = self.view.frame.size;
    
    // サブスクロールビューの設定
    _subScrollView.frame = rect;
    
    // サブスクロールビューのスケールをリセットする
    _subScrollView.zoomScale = 1.0f;
    _imageView1.transform = CGAffineTransformIdentity;
    
    // 中央のimage viewの設定
    rect.origin = CGPointZero;
    rect.size = image.size;
    _imageView1.frame = rect;
    _imageView1.image = image;
    [image release];
    
    // サブスクロールビューのコンテントサイズを設定する
    _subScrollView.contentSize = image.size;
    
    // サブスクロールビューのスケールを設定する
    float   hScale, vScale, minScale;
    hScale = CGRectGetWidth(_subScrollView.bounds) / image.size.width;
    vScale = CGRectGetHeight(_subScrollView.bounds) / image.size.height;
    minScale = MIN(hScale, vScale);
    _subScrollView.minimumZoomScale = minScale;
    _subScrollView.maximumZoomScale = 1.0f;
    _subScrollView.zoomScale = minScale;
    
    //
    // 右側のimage viewを更新
    //
    
    // 最後の画像のとき
    if (_index >= _maxPage - 1) {
        rect = CGRectZero;
        
        image = nil;
    }
    // 最後の画像以外のとき
    else {
        // 右側のimage viewのframe
        rect.origin.x = CGRectGetMaxX(_subScrollView.frame) + 20.0f;
        rect.origin.y = 0;
        rect.size = self.view.frame.size;
        
        // 右側のimage viewの画像の読み込み
        fileName = [NSString stringWithFormat:@"%03ds", _index + 1];
        path = [[NSBundle mainBundle] pathForResource:fileName ofType:@"jpg"];
        image = [[UIImage alloc] initWithContentsOfFile:path];
    }
    
    // 右側のimage viewの設定
    _imageView2.frame = rect;
    _imageView2.image = image;
    [image release];
    
    //
    // メインスクロールビューの更新
    //
    
    // コンテントサイズとオフセットの設定
    CGSize  size;
    size.width = CGRectGetMaxX(_imageView2.frame) > 0 ? 
            CGRectGetMaxX(_imageView2.frame) + 20.0f : 
            CGRectGetMaxX(_subScrollView.frame) + 20.0f;
    size.height = 0;
    _mainScrollView.contentSize = size;
    _mainScrollView.contentOffset = _subScrollView.frame.origin;
}

- (void)_updateImageCenter
{
    // 画像の表示されている大きさを取得
    UIImage*    image;
    CGSize      imageSize;
    image = _imageView1.image;
    imageSize = image.size;
    imageSize.width *= _subScrollView.zoomScale;
    imageSize.height *= _subScrollView.zoomScale;
    
    // サブスクロールビューの大きさを取得
    CGRect  bounds;
    bounds = _subScrollView.bounds;
    
    // イメージビューの中央の座標を計算
    CGPoint point;
    point.x = imageSize.width * 0.5f;
    if (imageSize.width < CGRectGetWidth(bounds)) {
        point.x += (CGRectGetWidth(bounds) - imageSize.width) * 0.5f;
    }
    point.y = imageSize.height * 0.5f;
    if (imageSize.height < CGRectGetHeight(bounds)) {
        point.y += (CGRectGetHeight(bounds) - imageSize.height) * 0.5f;
    }
    _imageView1.center = point;
}

//--------------------------------------------------------------//
#pragma mark -- UIScrollViewDelegate --
//--------------------------------------------------------------//

- (void)scrollViewDidEndDragging:(UIScrollView*)scrollView 
        willDecelerate:(BOOL)decelerate
{
    // メインスクロールビューの場合
    if (scrollView == _mainScrollView) {
        if (!decelerate) {
            // 画像の更新
            [self _renewImages];
        }
    }
}

- (void)scrollViewDidEndDecelerating:(UIScrollView*)scrollView
{
    // メインスクロールビューの場合
    if (scrollView == _mainScrollView) {
        // 画像の更新
        [self _renewImages];
    }
}

- (UIView*)viewForZoomingInScrollView:(UIScrollView*)scrollView
{
    // サブスクロールビューの場合
    if (scrollView == _subScrollView) {
        // 中央のimage viewを使う
        return _imageView1;
    }
    
    return nil;
}

- (void)scrollViewDidZoom:(UIScrollView*)scrollView
{
    // Update appearance
    [self _updateImageCenter];
}

- (void)scrollViewDidEndZooming:(UIScrollView*)scrollView 
        withView:(UIView*)view atScale:(float)scale
{
    [self _updateImageCenter];
}

@end
